@@ -85,6 +85,7 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  replacePNGData, NULL, 0)
     PHP_ME(swfed,  replaceGIFData, NULL, 0)
     PHP_ME(swfed,  replaceBitmapData, NULL, 0)
+    PHP_ME(swfed,  convertBitmapDataToJpegTag, NULL, 0)
     PHP_ME(swfed,  applyShapeMatrixFactor, NULL, 0)
     PHP_ME(swfed,  applyShapeRectFactor, NULL, 0)
     PHP_ME(swfed,  getSoundData, NULL, 0)
@@ -107,7 +108,7 @@ zend_function_entry swfed_functions[] = {
 };
 /* }}} */
 
-#define SWFED_VERSION "0.49"
+#define SWFED_VERSION "0.50"
 
 /* {{{ swfed_module_entry
  */
@@ -1196,6 +1197,17 @@ PHP_METHOD(swfed, replaceBitmapData) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, convertBitmapDataToJpegTag) {
+    swf_object_t *swf = NULL;
+    int ret;
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    ret = swf_object_convert_bitmapdata_tojpegtag(swf);
+    if (ret) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, applyShapeMatrixFactor) {
     long shape_id = 0;
     double scale_x = 1, scale_y = 1, rotate_rad = 0;
@@ -1307,14 +1319,19 @@ PHP_METHOD(swfed, getEditString) {
     swf_object_t *swf = NULL;
     char *data = NULL, *new_buff = NULL;
     int str_len = 0;
+    int error = 0;
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
                               &var_name, &var_name_len) == FAILURE) {
         RETURN_FALSE;
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    data = swf_object_get_editstring(swf, var_name, var_name_len);
+    data = swf_object_get_editstring(swf, var_name, var_name_len, &error);
     if (data == NULL) {
-        RETURN_FALSE;
+        if (error) {
+            RETURN_FALSE;
+        } else {
+            RETURN_NULL();
+        }
     }
     str_len = strlen(data);
     new_buff = emalloc(str_len);
