@@ -21,12 +21,18 @@ typedef struct swf_matrix_ {
     signed long rotate_skew0; // : rotate_bits; n.16 fixed point
     signed long rotate_skew1; // : rotate_bits; n.16 fixed point
 
-    signed translate_x ; // : f_rotate_bits;
-    signed translate_y ; // : f_rotate_bits;
+    unsigned translate_bits; // : f_rotate_bits;
+    signed translate_x;      // : f_rotate_bits;
+    signed translate_y;      // : f_rotate_bits;
 } swf_matrix_t;
 
 extern int swf_matrix_parse(bitstream_t *bs, swf_matrix_t *matrix);
 extern int swf_matrix_build(bitstream_t *bs, swf_matrix_t *matrix);
 extern int swf_matrix_print(swf_matrix_t *matrix, int indent_depth);
 
+extern int swf_matrix_apply_factor(swf_matrix_t *matrix,
+                                   double scale_x, double scale_y,
+                                   double rotate_rad,
+                                   signed int trans_x, signed int trans_y);
+
 #endif /* __SWF_MATRIX_H__ */
