@@ -80,6 +80,8 @@ extern int swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
 extern int swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
                                     unsigned char *gif_data,
                                     unsigned long gif_data_len);
+extern int swf_tag_convert_bitmap_data_tojpegtag(swf_tag_t *tag);
+
 /* sound */
 extern unsigned char *swf_tag_get_sound_data(swf_tag_t *tag,
                                              unsigned long *length,
@@ -92,6 +94,7 @@ extern int swf_tag_replace_melo_data(swf_tag_t *tag, int sound_id,
 extern char *swf_tag_get_edit_string(swf_tag_t *tag,
                                      char *variable_name,
                                      int variable_name_len,
+                                     int *error,
                                      struct swf_object_ *swf);
 extern int swf_tag_replace_edit_string(swf_tag_t *tag,
                                        char *variable_name,
